"""Provides messages that apply to the whole application."""

##############################################################################
# Local imports.
from .main import (
    GotoPEP,
    ShowAuthor,
    ShowPythonVersion,
    ShowStatus,
    ShowType,
    VisitPEP,
)

##############################################################################
# Exports.
__all__ = [
    "GotoPEP",
    "ShowAuthor",
    "ShowPythonVersion",
    "ShowStatus",
    "ShowType",
    "VisitPEP",
]

### __init__.py ends here
